/*
 *
 * PSX Memory Card Information v1.4 17/01/00 Richard Davies 
 * <mailto:richard@debaser.force9.co.uk>
 *
 * Revision History:
 * v1.4 - 17/01/00 Win32 / Win32 DLL support and bug fixes
 * v1.3 - 21/12/99 Linux support and bug fixes
 * v1.0 - 17/07/99 Initial release.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include "psxpbcfg.h"

#ifdef _PSXPB_WIN32DLL_
	#include <conio.h>
	#define	getch() _getch()
#endif

#ifdef _PSXPB_WIN32_
	#include <conio.h>
	#define	getch() _getch()
#endif

#ifdef _PSXPB_LINUX_
	#define	getch() getchar()
#endif


/*
 *
 * Prints the info associated with block from the directory
 *
 */
/*void psx_mcb_print_dir(PSX_MCB_INFO_DIR mcb_info_dir)
{
   if (mcb_info_dir.read)
   {
      printf("Filename:     '%s'\n", mcb_info_dir.filename);
      printf("Product Code: '%s'\n", mcb_info_dir.code);
      printf("Territory:    ");
      switch(mcb_info_dir.territory)
      {
         case 'E' : printf("European\n"); break;
         case 'A' : printf("American\n"); break;
         case 'I' : printf("Japanese\n"); break;
         default  : printf("Unknown\n");  break;
      }
      printf("Bytes Used:   %d\n", mcb_info_dir.bytes);
      printf("Status:       ");
      switch(mcb_info_dir.state)
      {
         case PSX_MCB_STATE_OK       : printf("OK\n");       break;
         case PSX_MCB_STATE_DELETED  : printf("Deleted\n");  break;
         case PSX_MCB_STATE_RESERVED : printf("Reserved\n"); break;
         default                     : printf("Unknown\n");  break;
      }
      printf("Link Type:    ");
      switch(mcb_info_dir.linktype)
      {
         case PSX_MCB_LTYPE_NONE     : printf("None\n");     break;
         case PSX_MCB_LTYPE_FIRST    : printf("First\n");    break;
         case PSX_MCB_LTYPE_MIDDLE   : printf("Middle\n");   break;
         case PSX_MCB_LTYPE_LAST     : printf("Last\n");     break;
         case PSX_MCB_LTYPE_RESERVED : printf("Reserved\n"); break;
         default                     : printf("Unknown\n");  break;
      }
      printf("Next Block:   ");
      if (mcb_info_dir.next < 15)
         printf("%d\n", mcb_info_dir.next+1);
      else
         printf("None\n");
   }
   else
      printf("Read Error!\n");
}*/

/*
 *
 * Prints the info associated with block from it's data
 *
 */
/*void psx_mcb_print_dat(PSX_MCB_INFO_DAT mcb_info_dat)
{
   if(mcb_info_dat.read)
   {
      printf("Name:         '%s'\n", mcb_info_dat.name);
      printf("Blocks Used:  %d\n", mcb_info_dat.blocks);
      printf("Icon:         ");
      if(mcb_info_dat.icon_valid & PSX_MCB_ICON_VALID)
         printf("%d frame%s\n", mcb_info_dat.icon_frames, (mcb_info_dat.icon_frames > 1) ? "s" : "");
      else
         printf("Invalid\n");
   }
   else
      printf("Read Error!\n");
}*/

/*
 *
 * Prints the info associated with block from the directory and it's data
 *
 */
void psx_mcb_print_info(PSX_MCB_INFO mcb_info)
{
   if(mcb_info.read)
   {
      printf("Name:         '%s'\n", mcb_info.name);
      printf("Filename:     '%s'\n", mcb_info.filename);
      printf("Product Code: '%s'\n", mcb_info.code);
      printf("Territory:    ");
      switch(mcb_info.territory)
      {
         case 'E' : printf("European\n"); break;
         case 'A' : printf("American\n"); break;
         case 'I' : printf("Japanese\n"); break;
         default  : printf("Unknown\n");  break;
      }
      printf("Status:       ");
      switch(mcb_info.state)
      {
         case PSX_MCB_STATE_OK       : printf("OK\n");       break;
         case PSX_MCB_STATE_DELETED  : printf("Deleted\n");  break;
         case PSX_MCB_STATE_RESERVED : printf("Reserved\n"); break;
         default                     : printf("Unknown\n");  break;
      }
      printf("Blocks Used:  %d\n", mcb_info.blocks);
      printf("Bytes Used:   %d\n", mcb_info.bytes);
      printf("Link Type:    ");
      switch(mcb_info.linktype)
      {
         case PSX_MCB_LTYPE_NONE     : printf("None\n");     break;
         case PSX_MCB_LTYPE_FIRST    : printf("First\n");    break;
         case PSX_MCB_LTYPE_MIDDLE   : printf("Middle\n");   break;
         case PSX_MCB_LTYPE_LAST     : printf("Last\n");     break;
         case PSX_MCB_LTYPE_RESERVED : printf("Reserved\n"); break;
         default                     : printf("Unknown\n");  break;
      }
      printf("Next Block:   ");
      if (mcb_info.next < 15)
         printf("%d\n", mcb_info.next+1);
      else
         printf("None\n");
      printf("Icon:         ");
      if(mcb_info.icon_valid & PSX_MCB_ICON_VALID)
         printf("%d frame%s\n", mcb_info.icon_frames, (mcb_info.icon_frames > 1) ? "s" : "");
      else
         printf("Invalid\n");
   }
   else
      printf("Read Error!\n");
}

int main(int argc, char *argv[])
{
   int i, base, conport, tap, delay, block, linkblock;
   PSX_MCB_INFO mcbs[15];

   printf("PSX Memory Card Information v1.4 17/01/00 Richard Davies\n");
   printf("<mailto:richard@debaser.force9.co.uk>\n\n");

   printf("Use: psxmci [-lpt (1..3)] [-con (1..%d)] [-tap (A..%c)] [-del (1..%d)]\n", PSX_MAX_CONPORTS, PSX_MAX_TAPS + ('A' - 1), PSX_MAX_DELAY);
   printf("Settings also held in 'psxpblib.cfg'\n\n");

#ifdef _PSXPB_WIN32DLL_
   if (pw32dll() == -1)
   {
	   printf("Unable to load 'psxpbdll.dll'\n");
	   return 0;
   }
#endif

   base = -1;
   conport = -1;
   tap = 1;
   delay = 3;

   psxpb_cfg(&base, &conport, &tap, &delay);
   
   psxpb_cmd(&base, &conport, &tap, &delay, argc, argv);
   
   /* check parameters */
   if (base == -1)
   {
      printf("Error! You need to specify which parallel port to use!\n");
      return 0;
   }
   if (conport == -1)
   {
      printf("Error! You need to specify which controller port to use!\n");
      return 0;
   }
   
   /* report parameters */
   printf("Parallel port: base 0x%x\n", base); 
   printf("Controller port: %d\n", conport);
   printf("Controller port tap: %c\n", tap + ('A' - 1));
   printf("Scan delay: %d\n\n", delay);
   
   printf("Any key to continue...\n\n");
   if (getch() == 27)
   {
      return 0;
   }

   for (block = 1; block < 16; block++)
   {
      mcbs[block-1].scanned = 0;
   }
   
   /* Memory Card Scan */
   printf("Scanning Memory Card...\n\n");
   for (block = 1; block < 16; block++)
   {
      if (!mcbs[block-1].scanned)
      {
	 PSX_MCB_INFO* tmp;
	 
	 if(!(tmp = psx_mcb_read_info(base, conport, tap, delay, block))){
		 printf("Memory Card not found!\n");
		 return 0;
	 }
         mcbs[block-1] = *tmp;
         if (mcbs[block-1].read)
         {
            if ((mcbs[block-1].linktype != PSX_MCB_LTYPE_MIDDLE) && (mcbs[block-1].linktype != PSX_MCB_LTYPE_LAST))
            {
               mcbs[block-1].scanned = 1;
               printf("Block %2d: ", block);
               switch(mcbs[block-1].state)
               {
                  case PSX_MCB_STATE_DELETED  : printf("D "); break;
                  case PSX_MCB_STATE_RESERVED : printf("R "); break;
                  case PSX_MCB_STATE_OK       : printf("A "); break;
                  default                     : printf("U "); break;
               }
               printf("'%s' ", mcbs[block-1].name);
               if (mcbs[block-1].next < 15)
               {
                  printf("**Part 1 of %d**\n", mcbs[block-1].blocks);
                  linkblock = block;
                  for(i = 1; i < mcbs[block-1].blocks; i++)
                  {
                     mcbs[mcbs[linkblock-1].next] = *(psx_mcb_read_info(base, conport, tap, delay, mcbs[linkblock-1].next+1));
                     if (mcbs[mcbs[linkblock-1].next].read)
                     {
                        if ((mcbs[mcbs[linkblock-1].next].linktype == PSX_MCB_LTYPE_MIDDLE) || (mcbs[mcbs[linkblock-1].next].linktype == PSX_MCB_LTYPE_LAST))
                        {
                           mcbs[mcbs[linkblock-1].next].scanned = 1;
                           printf("Block %2d: ", mcbs[linkblock-1].next+1);
                           switch(mcbs[mcbs[linkblock-1].next].state)
                           {
                              case PSX_MCB_STATE_DELETED  : printf("D "); break;
                              case PSX_MCB_STATE_RESERVED : printf("R "); break;
                              case PSX_MCB_STATE_OK       : printf("A "); break;
                              default                     : printf("U "); break;
                           }
                           printf("'%s' ", mcbs[block-1].name);
                           printf("**Part %d of %d**\n", i+1, mcbs[block-1].blocks);
                        }
                        else
                        {
                           printf("Block %2d: '%s' Error! Part %d missing!\n", mcbs[linkblock-1].next+1, mcbs[block-1].name, i);
                           i = mcbs[block-1].blocks;
                        }  
                     }
                     else
                     {
                        mcbs[mcbs[linkblock-1].next].scanned = 1;
                        printf("Block %2d: '%s' **Part %d Read Error!**\n", mcbs[linkblock-1].next+1,  mcbs[block-1].name, i);
                        i = mcbs[block-1].blocks;
                        return 0;
                     }
                     linkblock = mcbs[linkblock-1].next + 1;                     
                  }
               }
               else
                  printf("\n");
            }
         }
         else
         {
            printf("Block %2d: **Read Error!**\n", block);
            return 0;
         }
      }
   }
   
   /* Block by Block info */
   for (block = 1; block < 16; block++)
   {
      printf("\nAny key to continue...\n\n");
      if (getch() == 27)
      {
         return 0;
      }
      printf("Block %d...\n\n", block);
      /*mcbs[block-1] = *(psx_mcb_read_info(base, conport, tap, block));*/
      psx_mcb_print_info(mcbs[block-1]);
   }
	
   return 0;
}
