/*
 *
 * PSX Memory Card Reader v1.4 17/01/00 Richard Davies 
 * <mailto:richard@debaser.force9.co.uk>
 *
 * Revision History:
 * v1.4 - 17/01/00 Win32 / Win32 DLL support and bug fixes
 * v1.3 - 21/12/99 Linux support and bug fixes
 * v1.0 - 17/07/99 Initial release.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include "psxpbcfg.h"

#ifdef _PSXPB_WIN32DLL_
	#include <conio.h>
	#define	getch() _getch()
#endif

#ifdef _PSXPB_WIN32_
	#include <conio.h>
	#define	getch() _getch()
#endif

#ifdef _PSXPB_LINUX_
	#define	getch() getchar()
#endif


int main(int argc, char *argv[])
{
   int base, conport, tap, delay, block;
   unsigned char *data;
   char filename[13];
   FILE *f;

   printf("PSX Memory Card Reader v1.4 17/01/00 Richard Davies\n");
   printf("<mailto:richard@debaser.force9.co.uk>\n\n");

   printf("Use: psxmcr [-lpt (1..3)] [-con (1..%d)] [-tap (A..%c)] [-del (1..%d)]\n", PSX_MAX_CONPORTS, PSX_MAX_TAPS + ('A' - 1), PSX_MAX_DELAY);
   printf("Settings also held in 'psxpblib.cfg'\n\n");

#ifdef _PSXPB_WIN32DLL_
   if (pw32dll() == -1)
   {
	   printf("Unable to load 'psxpbdll.dll'\n");
	   return 0;
   }
#endif

   base = -1;
   conport = -1;
   tap = 1;
   delay = 3;

   psxpb_cfg(&base, &conport, &tap, &delay);
   
   psxpb_cmd(&base, &conport, &tap, &delay, argc, argv);
   
   /* check parameters */
   if (base == -1)
   {
      printf("Error! You need to specify which parallel port to use!\n");
      return 0;
   }
   if (conport == -1)
   {
      printf("Error! You need to specify which controller port to use!\n");
      return 0;
   }
   
   /* report parameters */
   printf("Parallel port: base 0x%x\n", base); 
   printf("Controller port: %d\n", conport);
   printf("Controller port tap: %c\n", tap + ('A' - 1));
   printf("Scan delay: %d\n\n", delay);
   
   strcpy(filename, "temp.mc");
     
   f = fopen(filename, "rb");
      
   if (f)
   {
      fclose(f);

	  if (remove(filename))
	  {
	     printf("\nUnable to remove '%s'.\n", filename);
	     return 0;
	  }
   }

   strcpy(filename, "~temp.mc");
     
   f = fopen(filename, "wb");
      
   if (!f)
   {
      printf("Error creating '%s'.\n", filename);                      
      fclose(f);
      return 0;
   }

   for (block = 0; block < 16; block++)
   {    
      printf("Reading block %d from Memory Card...\n", block);
   
      data = psx_memcard_read_block(base, conport, tap, delay, block);

      if (data != NULL)
      {
         printf("Done, Writing block %d to '%s'.\n", block, filename);
         if (fwrite(data, 1, 8192, f) == 8192)
            printf("Done.\n\n");
         else
            printf("Failed! Error writing block %d to '%s'.\n", block, filename);
      }
      else
      {
         printf("Failed! Error reading block %d from Memory Card.\n\n", block);
         fclose(f);
         return 0;
      }
   }
	
   fclose(f);
   
   printf("Renaming '%s' to 'temp.mc'...\n", filename);
   rename (filename, "temp.mc");
   printf("Done.\n");
   
   return 0;
}
