/*
 *
 * PSX Memory Card Speed Analyser v1.4 17/01/00 Richard Davies 
 * <mailto:richard@debaser.force9.co.uk>
 *
 * Revision History:
 * v1.4 - 17/01/00 Win32 / Win32 DLL support and bug fixes
 * v1.3 - 21/12/99 Linux support and bug fixes
 * v1.0 - 17/07/99 Initial release.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "psxpbcfg.h"

#ifdef _PSXPB_WIN32DLL_
	#define	uclock clock
	#define	uclock_t clock_t
	#define UCLOCKS_PER_SEC CLOCKS_PER_SEC
#endif

#ifdef _PSXPB_WIN32_
	#define	uclock clock
	#define	uclock_t clock_t
	#define UCLOCKS_PER_SEC CLOCKS_PER_SEC
#endif

#ifdef _PSXPB_LINUX_
	#define	uclock clock
	#define	uclock_t clock_t
	#define UCLOCKS_PER_SEC CLOCKS_PER_SEC
#endif


int main(int argc, char *argv[])
{
   int base, conport, tap, delay, block;
   unsigned char *data;
   double time;
   uclock_t start, finish;
   
   printf("PSX Memory Card Speed Analyser v1.4 17/01/00 Richard Davies\n");
   printf("<mailto:richard@debaser.force9.co.uk>\n\n");

   printf("Use: psxmcs [-lpt (1..3)] [-con (1..%d)] [-tap (A..%c)] [-del (1..%d)]\n", PSX_MAX_CONPORTS, PSX_MAX_TAPS + ('A' - 1), PSX_MAX_DELAY);
   printf("Settings also held in 'psxpblib.cfg'\n\n");

#ifdef _PSXPB_WIN32DLL_
   if (pw32dll() == -1)
   {
	   printf("Unable to load 'psxpbdll.dll'\n");
	   return 0;
   }
#endif

   base = -1;
   conport = -1;
   tap = 1;
   delay = 3;

   psxpb_cfg(&base, &conport, &tap, &delay);
   
   psxpb_cmd(&base, &conport, &tap, &delay, argc, argv);
   
   /* check parameters */
   if (base == -1)
   {
      printf("Error! You need to specify which parallel port to use!\n");
      return 0;
   }
   if (conport == -1)
   {
      printf("Error! You need to specify which controller port to use!\n");
      return 0;
   }
   
   /* report parameters */
   printf("Parallel port: base 0x%x\n", base); 
   printf("Controller port: %d\n", conport);
   printf("Controller port tap: %c\n", tap + ('A' - 1));
   printf("Scan delay: %d\n\n", delay);

   printf("Timing entire Memory Card read...\n\n");
   
   start = uclock();
   
   for (block = 0; block < 16; block++)
   {    
      printf("Reading block %d from Memory Card...\n", block);
   
      data = psx_memcard_read_block(base, conport, tap, delay, block);

      if (data != NULL)
      {
         printf("Done.\n\n");
      }
      else
      {
         printf("Failed! Error reading block %d from Memory Card.\n\n", block);
         return 0;
      }
   }
   
   finish = uclock();
   
   time = ((float)(finish - start))/((float)UCLOCKS_PER_SEC);
   
   printf("Time taken: %f seconds, at %f KBytes per second.\n\n", time, 128.0/time);
	
   return 0;
}
