/*
 *
 * PSX Memory Card Writer v1.4 17/01/00 Richard Davies 
 * <mailto:richard@debaser.force9.co.uk>
 *
 * Revision History:
 * v1.4 - 17/01/00 Win32 / Win32 DLL support and bug fixes
 * v1.3 - 21/12/99 Linux support and bug fixes
 * v1.0 - 17/07/99 Initial release.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include "psxpbcfg.h"


int main(int argc, char *argv[])
{
   int base, conport, tap, delay, block, length;
   unsigned char data[8192];
   char filename[13];
   FILE *f;

   printf("PSX Memory Card Writer v1.4 17/01/00 Richard Davies\n");
   printf("<mailto:richard@debaser.force9.co.uk>\n\n");

   printf("Use: psxmcw [-lpt (1..3)] [-con (1..%d)] [-tap (A..%c)] [-del (1..%d)]\n", PSX_MAX_CONPORTS, PSX_MAX_TAPS + ('A' - 1), PSX_MAX_DELAY);
   printf("Settings also held in 'psxpblib.cfg'\n\n");

#ifdef _PSXPB_WIN32DLL_
   if (pw32dll() == -1)
   {
	   printf("Unable to load 'psxpbdll.dll'\n");
	   return 0;
   }
#endif

   base = -1;
   conport = -1;
   tap = 1;
   delay = 3;

   psxpb_cfg(&base, &conport, &tap, &delay);
   
   psxpb_cmd(&base, &conport, &tap, &delay, argc, argv);
   
   /* check parameters */
   if (base == -1)
   {
      printf("Error! You need to specify which parallel port to use!\n");
      return 0;
   }
   if (conport == -1)
   {
      printf("Error! You need to specify which controller port to use!\n");
      return 0;
   }
   
   /* report parameters */
   printf("Parallel port: base 0x%x\n", base); 
   printf("Controller port: %d\n", conport);
   printf("Controller port tap: %c\n", tap + ('A' - 1));
   printf("Scan delay: %d\n\n", delay);
   
   /* open Memory Card file */
   strcpy(filename, "temp.mc");
   
   printf("Opening Memory Card file '%s'...\n", filename);
   
   f = fopen(filename, "rb");
   
   if (f == NULL)
   {
      printf("Failed! Error opening '%s'.\n", filename);
      return 0;
   }
   else
      printf("Done.\n\n");

   /* determine length of file */
   printf("Determining '%s' file length...\n", filename);
   
   if (fseek (f, 0L, SEEK_END) != 0)
   {
      printf("Failed! Error reading file.\n");
      fclose(f);
      return 0;
   }

   length = ftell(f);
   if (length != 131072)
   {
      printf("Failed! Expected 131072 bytes, found %d bytes.\n", length);
      fclose(f);
      return 0;
   }
   
   if (fseek(f, 0, 0) != 0)
   {
      printf("Failed! Error reading file.\n");
      fclose(f);
      return 0;
   }
   else
      printf("Done.\n\n");

   /* Write Memory Card file to Memory Card */
   for (block = 0; block < 16; block++)
   {    
      printf("Reading block %d from '%s'...\n", block, filename);
   
      length = fread(data, 1, 8192, f);
      
      if (length == 8192)
      {
         printf("Done, Writing block %d to Memory Card (%s).\n",
			 block, filename);
         
         if (psx_memcard_write_block(base, conport, tap, delay, block, data) ==
			 -1)
         {
            printf("Failed! Error writing block %d to Memory Card (%s).\n",
			    block, filename);
            fclose(f);
            return 0; 
         }
         else
            printf("Done.\n\n");      	
      }
      else
      {
         printf("Failed! Error reading block %d from '%s'.\n\n", block, filename);
         fclose(f);
         return 0;
      }
   }
   
   /* close and delete file */
   fclose(f);
   
   printf("Deleting '%s'...\n", filename);
   remove(filename);
   printf("Done.\n");
   
   return 0;
}
